var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { publicKeyExt } from './utils';
import * as RaceCore from '@race/sdk-core';
import { deserialize, serialize, field, option, array, struct } from '@race/borsh';
export class PlayerState {
    constructor(fields) {
        Object.assign(this, fields);
    }
    serialize() {
        return serialize(this);
    }
    static deserialize(data) {
        return deserialize(PlayerState, data);
    }
    generalize(addr) {
        return new RaceCore.PlayerProfile({
            addr: addr.toBase58(),
            nick: this.nick,
            pfp: this.pfpKey?.toBase58(),
        });
    }
}
__decorate([
    field('bool')
], PlayerState.prototype, "isInitialized", void 0);
__decorate([
    field('string')
], PlayerState.prototype, "nick", void 0);
__decorate([
    field(option(publicKeyExt))
], PlayerState.prototype, "pfpKey", void 0);
export class Vote {
    constructor(fields) {
        Object.assign(this, fields);
    }
    generalize() {
        return new RaceCore.Vote({
            voter: this.voterKey.toBase58(),
            votee: this.voteeKey.toBase58(),
            voteType: this.voteType,
        });
    }
}
__decorate([
    field(publicKeyExt)
], Vote.prototype, "voterKey", void 0);
__decorate([
    field(publicKeyExt)
], Vote.prototype, "voteeKey", void 0);
__decorate([
    field('u8')
], Vote.prototype, "voteType", void 0);
export class ServerJoin {
    constructor(fields) {
        Object.assign(this, fields);
    }
    generalize() {
        return new RaceCore.ServerJoin({
            addr: this.key.toBase58(),
            endpoint: this.endpoint,
            accessVersion: this.accessVersion,
            verifyKey: this.verifyKey,
        });
    }
}
__decorate([
    field(publicKeyExt)
], ServerJoin.prototype, "key", void 0);
__decorate([
    field('string')
], ServerJoin.prototype, "endpoint", void 0);
__decorate([
    field('u64')
], ServerJoin.prototype, "accessVersion", void 0);
__decorate([
    field('string')
], ServerJoin.prototype, "verifyKey", void 0);
export class PlayerJoin {
    constructor(fields) {
        Object.assign(this, fields);
    }
    generalize() {
        return new RaceCore.PlayerJoin({
            addr: this.key.toBase58(),
            position: this.position,
            balance: this.balance,
            accessVersion: this.accessVersion,
            verifyKey: this.verifyKey,
        });
    }
}
__decorate([
    field(publicKeyExt)
], PlayerJoin.prototype, "key", void 0);
__decorate([
    field('u64')
], PlayerJoin.prototype, "balance", void 0);
__decorate([
    field('u16')
], PlayerJoin.prototype, "position", void 0);
__decorate([
    field('u64')
], PlayerJoin.prototype, "accessVersion", void 0);
__decorate([
    field('string')
], PlayerJoin.prototype, "verifyKey", void 0);
export class GameState {
    constructor(fields) {
        Object.assign(this, fields);
    }
    serialize() {
        return serialize(this);
    }
    static deserialize(data) {
        return deserialize(GameState, data);
    }
    generalize(addr) {
        return new RaceCore.GameAccount({
            addr: addr.toBase58(),
            title: this.title,
            bundleAddr: this.bundleKey.toBase58(),
            ownerAddr: this.ownerKey.toBase58(),
            tokenAddr: this.tokenKey.toBase58(),
            deposits: [],
            minDeposit: this.minDeposit,
            maxDeposit: this.maxDeposit,
            transactorAddr: this.transactorKey?.toBase58(),
            accessVersion: this.accessVersion,
            settleVersion: this.settleVersion,
            maxPlayers: this.maxPlayers,
            players: this.players.map(p => p.generalize()),
            servers: this.servers.map(s => s.generalize()),
            dataLen: this.dataLen,
            data: this.data,
            votes: this.votes.map(v => v.generalize()),
            unlockTime: this.unlockTime,
        });
    }
}
__decorate([
    field('bool')
], GameState.prototype, "isInitialized", void 0);
__decorate([
    field('string')
], GameState.prototype, "title", void 0);
__decorate([
    field(publicKeyExt)
], GameState.prototype, "bundleKey", void 0);
__decorate([
    field(publicKeyExt)
], GameState.prototype, "stakeKey", void 0);
__decorate([
    field(publicKeyExt)
], GameState.prototype, "ownerKey", void 0);
__decorate([
    field(publicKeyExt)
], GameState.prototype, "tokenKey", void 0);
__decorate([
    field('u64')
], GameState.prototype, "minDeposit", void 0);
__decorate([
    field('u64')
], GameState.prototype, "maxDeposit", void 0);
__decorate([
    field(option(publicKeyExt))
], GameState.prototype, "transactorKey", void 0);
__decorate([
    field('u64')
], GameState.prototype, "accessVersion", void 0);
__decorate([
    field('u64')
], GameState.prototype, "settleVersion", void 0);
__decorate([
    field('u16')
], GameState.prototype, "maxPlayers", void 0);
__decorate([
    field(array(struct(PlayerJoin)))
], GameState.prototype, "players", void 0);
__decorate([
    field(array(struct(ServerJoin)))
], GameState.prototype, "servers", void 0);
__decorate([
    field('u32')
], GameState.prototype, "dataLen", void 0);
__decorate([
    field('u8-array')
], GameState.prototype, "data", void 0);
__decorate([
    field(array(struct(Vote)))
], GameState.prototype, "votes", void 0);
__decorate([
    field(option('u64'))
], GameState.prototype, "unlockTime", void 0);
export class GameReg {
    constructor(fields) {
        Object.assign(this, fields);
    }
    generalize() {
        return new RaceCore.GameRegistration({
            title: this.title,
            addr: this.gameKey.toBase58(),
            bundleAddr: this.bundleKey.toBase58(),
            regTime: this.regTime,
        });
    }
}
__decorate([
    field('string')
], GameReg.prototype, "title", void 0);
__decorate([
    field(publicKeyExt)
], GameReg.prototype, "gameKey", void 0);
__decorate([
    field(publicKeyExt)
], GameReg.prototype, "bundleKey", void 0);
__decorate([
    field('u64')
], GameReg.prototype, "regTime", void 0);
export class RegistryState {
    constructor(fields) {
        Object.assign(this, fields);
    }
    serialize() {
        return serialize(this);
    }
    static deserialize(data) {
        return deserialize(RegistryState, data);
    }
    generalize(addr) {
        return new RaceCore.RegistrationAccount({
            addr: addr.toBase58(),
            isPrivate: this.isPrivate,
            size: this.size,
            owner: this.ownerKey.toBase58(),
            games: this.games.map(g => g.generalize()),
        });
    }
}
__decorate([
    field('bool')
], RegistryState.prototype, "isInitialized", void 0);
__decorate([
    field('bool')
], RegistryState.prototype, "isPrivate", void 0);
__decorate([
    field('u16')
], RegistryState.prototype, "size", void 0);
__decorate([
    field(publicKeyExt)
], RegistryState.prototype, "ownerKey", void 0);
__decorate([
    field(array(struct(GameReg)))
], RegistryState.prototype, "games", void 0);
export class ServerState {
    constructor(fields) {
        Object.assign(this, fields);
    }
    serialize() {
        return serialize(this);
    }
    static deserialize(data) {
        return deserialize(this, data);
    }
    generalize() {
        return new RaceCore.ServerAccount({
            addr: this.ownerKey.toBase58(),
            endpoint: this.endpoint,
        });
    }
}
__decorate([
    field('bool')
], ServerState.prototype, "isInitialized", void 0);
__decorate([
    field(publicKeyExt)
], ServerState.prototype, "key", void 0);
__decorate([
    field(publicKeyExt)
], ServerState.prototype, "ownerKey", void 0);
__decorate([
    field('string')
], ServerState.prototype, "endpoint", void 0);
