import { PublicKey } from '@solana/web3.js';
import * as RaceCore from '@race-foundation/sdk-core';
import { VoteType } from '@race-foundation/sdk-core';
export interface IPlayerState {
    isInitialized: boolean;
    nick: string;
    pfpKey?: PublicKey;
}
export interface IPlayerJoin {
    key: PublicKey;
    balance: bigint;
    position: number;
    accessVersion: bigint;
    verifyKey: string;
}
export interface IServerJoin {
    key: PublicKey;
    endpoint: string;
    accessVersion: bigint;
    verifyKey: string;
}
export interface IVote {
    voterKey: PublicKey;
    voteeKey: PublicKey;
    voteType: VoteType;
}
export interface IGameReg {
    title: string;
    gameKey: PublicKey;
    bundleKey: PublicKey;
    regTime: bigint;
}
export interface IRegistryState {
    isInitialized: boolean;
    isPrivate: boolean;
    size: number;
    ownerKey: PublicKey;
    games: IGameReg[];
}
export interface IGameState {
    isInitialized: boolean;
    title: string;
    bundleKey: PublicKey;
    stakeKey: PublicKey;
    ownerKey: PublicKey;
    tokenKey: PublicKey;
    minDeposit: bigint;
    maxDeposit: bigint;
    transactorKey: PublicKey | undefined;
    accessVersion: bigint;
    settleVersion: bigint;
    maxPlayers: number;
    players: IPlayerJoin[];
    servers: IServerJoin[];
    dataLen: number;
    data: Uint8Array;
    votes: IVote[];
    unlockTime: bigint | undefined;
}
export interface IServerState {
    isInitialized: boolean;
    key: PublicKey;
    ownerKey: PublicKey;
    endpoint: string;
}
export declare class PlayerState implements IPlayerState {
    isInitialized: boolean;
    nick: string;
    pfpKey?: PublicKey;
    constructor(fields: IPlayerState);
    serialize(): Uint8Array;
    static deserialize(data: Uint8Array): PlayerState;
    generalize(addr: PublicKey): RaceCore.PlayerProfile;
}
export declare class Vote implements IVote {
    voterKey: PublicKey;
    voteeKey: PublicKey;
    voteType: VoteType;
    constructor(fields: IVote);
    generalize(): RaceCore.Vote;
}
export declare class ServerJoin implements IServerJoin {
    key: PublicKey;
    endpoint: string;
    accessVersion: bigint;
    verifyKey: string;
    constructor(fields: IServerJoin);
    generalize(): RaceCore.ServerJoin;
}
export declare class PlayerJoin implements IPlayerJoin {
    key: PublicKey;
    balance: bigint;
    position: number;
    accessVersion: bigint;
    verifyKey: string;
    constructor(fields: IPlayerJoin);
    generalize(): RaceCore.PlayerJoin;
}
export declare class GameState implements IGameState {
    isInitialized: boolean;
    title: string;
    bundleKey: PublicKey;
    stakeKey: PublicKey;
    ownerKey: PublicKey;
    tokenKey: PublicKey;
    minDeposit: bigint;
    maxDeposit: bigint;
    transactorKey: PublicKey | undefined;
    accessVersion: bigint;
    settleVersion: bigint;
    maxPlayers: number;
    players: PlayerJoin[];
    servers: ServerJoin[];
    dataLen: number;
    data: Uint8Array;
    votes: Vote[];
    unlockTime: bigint | undefined;
    constructor(fields: IGameState);
    serialize(): Uint8Array;
    static deserialize(data: Uint8Array): GameState;
    generalize(addr: PublicKey): RaceCore.GameAccount;
}
export declare class GameReg implements IGameReg {
    title: string;
    gameKey: PublicKey;
    bundleKey: PublicKey;
    regTime: bigint;
    constructor(fields: IGameReg);
    generalize(): RaceCore.GameRegistration;
}
export declare class RegistryState implements IRegistryState {
    isInitialized: boolean;
    isPrivate: boolean;
    size: number;
    ownerKey: PublicKey;
    games: GameReg[];
    constructor(fields: IRegistryState);
    serialize(): Uint8Array;
    static deserialize(data: Uint8Array): RegistryState;
    generalize(addr: PublicKey): RaceCore.RegistrationAccount;
}
export declare class ServerState implements IServerState {
    isInitialized: boolean;
    key: PublicKey;
    ownerKey: PublicKey;
    endpoint: string;
    constructor(fields: IServerState);
    serialize(): Uint8Array;
    static deserialize(data: Uint8Array): ServerState;
    generalize(): RaceCore.ServerAccount;
}
