"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.publishGame = exports.join = exports.closeGameAccount = exports.createGameAccount = exports.createPlayerProfile = exports.PublishGameData = exports.JoinGameData = exports.CreateGameAccountData = exports.CloseGameAccountData = exports.CreatePlayerProfileData = exports.Instruction = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const constants_1 = require("./constants");
const borsh_1 = require("@race-foundation/borsh");
const buffer_1 = require("buffer");
// Instruction types
var Instruction;
(function (Instruction) {
    Instruction[Instruction["CreateGameAccount"] = 0] = "CreateGameAccount";
    Instruction[Instruction["CloseGameAccount"] = 1] = "CloseGameAccount";
    Instruction[Instruction["CreateRegistration"] = 2] = "CreateRegistration";
    Instruction[Instruction["CreatePlayerProfile"] = 3] = "CreatePlayerProfile";
    Instruction[Instruction["RegisterServer"] = 4] = "RegisterServer";
    Instruction[Instruction["Settle"] = 5] = "Settle";
    Instruction[Instruction["Vote"] = 6] = "Vote";
    Instruction[Instruction["ServeGame"] = 7] = "ServeGame";
    Instruction[Instruction["RegisterGame"] = 8] = "RegisterGame";
    Instruction[Instruction["UnregisterGame"] = 9] = "UnregisterGame";
    Instruction[Instruction["JoinGame"] = 10] = "JoinGame";
    Instruction[Instruction["PublishGame"] = 11] = "PublishGame";
})(Instruction = exports.Instruction || (exports.Instruction = {}));
// Instruction data definitations
class Serialize {
    serialize() {
        return buffer_1.Buffer.from((0, borsh_1.serialize)(this));
    }
}
class CreatePlayerProfileData extends Serialize {
    constructor(nick) {
        super();
        this.instruction = Instruction.CreatePlayerProfile;
        this.nick = nick;
    }
}
__decorate([
    (0, borsh_1.field)('u8')
], CreatePlayerProfileData.prototype, "instruction", void 0);
__decorate([
    (0, borsh_1.field)('string')
], CreatePlayerProfileData.prototype, "nick", void 0);
exports.CreatePlayerProfileData = CreatePlayerProfileData;
class CloseGameAccountData extends Serialize {
    constructor() {
        super();
        this.instruction = Instruction.CloseGameAccount;
    }
}
__decorate([
    (0, borsh_1.field)('u8')
], CloseGameAccountData.prototype, "instruction", void 0);
exports.CloseGameAccountData = CloseGameAccountData;
class CreateGameAccountData extends Serialize {
    constructor(params) {
        super();
        this.instruction = Instruction.CreateGameAccount;
        this.title = '';
        this.maxPlayers = 0;
        this.minDeposit = 0n;
        this.maxDeposit = 0n;
        this.data = Uint8Array.from([]);
        Object.assign(this, params);
    }
}
__decorate([
    (0, borsh_1.field)('u8')
], CreateGameAccountData.prototype, "instruction", void 0);
__decorate([
    (0, borsh_1.field)('string')
], CreateGameAccountData.prototype, "title", void 0);
__decorate([
    (0, borsh_1.field)('u16')
], CreateGameAccountData.prototype, "maxPlayers", void 0);
__decorate([
    (0, borsh_1.field)('u64')
], CreateGameAccountData.prototype, "minDeposit", void 0);
__decorate([
    (0, borsh_1.field)('u64')
], CreateGameAccountData.prototype, "maxDeposit", void 0);
__decorate([
    (0, borsh_1.field)('u8-array')
], CreateGameAccountData.prototype, "data", void 0);
exports.CreateGameAccountData = CreateGameAccountData;
class JoinGameData extends Serialize {
    constructor(amount, accessVersion, position, verifyKey) {
        super();
        this.instruction = Instruction.JoinGame;
        this.amount = amount;
        this.accessVersion = accessVersion;
        this.position = position;
        this.verifyKey = verifyKey;
    }
}
__decorate([
    (0, borsh_1.field)('u8')
], JoinGameData.prototype, "instruction", void 0);
__decorate([
    (0, borsh_1.field)('u64')
], JoinGameData.prototype, "amount", void 0);
__decorate([
    (0, borsh_1.field)('u64')
], JoinGameData.prototype, "accessVersion", void 0);
__decorate([
    (0, borsh_1.field)('u16')
], JoinGameData.prototype, "position", void 0);
__decorate([
    (0, borsh_1.field)('string')
], JoinGameData.prototype, "verifyKey", void 0);
exports.JoinGameData = JoinGameData;
class PublishGameData extends Serialize {
    constructor(uri, name, symbol) {
        super();
        this.instruction = Instruction.PublishGame;
        this.uri = uri;
        this.name = name;
        this.symbol = symbol;
    }
}
__decorate([
    (0, borsh_1.field)('u8')
], PublishGameData.prototype, "instruction", void 0);
__decorate([
    (0, borsh_1.field)('string')
], PublishGameData.prototype, "uri", void 0);
__decorate([
    (0, borsh_1.field)('string')
], PublishGameData.prototype, "name", void 0);
__decorate([
    (0, borsh_1.field)('string')
], PublishGameData.prototype, "symbol", void 0);
exports.PublishGameData = PublishGameData;
// Instruction helpers
function createPlayerProfile(ownerKey, profileKey, nick, pfpKey) {
    const data = new CreatePlayerProfileData(nick).serialize();
    return new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: profileKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: pfpKey || web3_js_1.PublicKey.default,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: constants_1.PROGRAM_ID,
        data: buffer_1.Buffer.from(data),
    });
}
exports.createPlayerProfile = createPlayerProfile;
function createGameAccount(opts) {
    const data = new CreateGameAccountData(opts).serialize();
    return new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: opts.ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: opts.gameAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: opts.stakeAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: opts.mint,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: spl_token_1.TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: opts.gameBundleKey,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: constants_1.PROGRAM_ID,
        data,
    });
}
exports.createGameAccount = createGameAccount;
function closeGameAccount(opts) {
    const { ownerKey, gameAccountKey, regAccountKey, gameStakeKey } = opts;
    const data = new CloseGameAccountData().serialize();
    let [pda, _] = web3_js_1.PublicKey.findProgramAddressSync([gameAccountKey.toBuffer()], constants_1.PROGRAM_ID);
    return new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: gameAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: regAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: gameStakeKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: pda,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: constants_1.PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: constants_1.PROGRAM_ID,
        data,
    });
}
exports.closeGameAccount = closeGameAccount;
async function join(opts) {
    const { playerKey, paymentKey, gameAccountKey, mint, stakeAccountKey, amount, accessVersion, position, verifyKey } = opts;
    const profileKey = await web3_js_1.PublicKey.createWithSeed(playerKey, constants_1.PLAYER_PROFILE_SEED, constants_1.PROGRAM_ID);
    let [pda, _] = web3_js_1.PublicKey.findProgramAddressSync([gameAccountKey.toBuffer()], constants_1.PROGRAM_ID);
    const data = new JoinGameData(amount, accessVersion, position, verifyKey).serialize();
    return new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: playerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: profileKey,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: paymentKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: gameAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: mint,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: stakeAccountKey,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: pda,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: spl_token_1.TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: constants_1.PROGRAM_ID,
        data,
    });
}
exports.join = join;
function publishGame(opts) {
    const { ownerKey, mint, uri, name, symbol } = opts;
    let [metadataPda] = web3_js_1.PublicKey.findProgramAddressSync([buffer_1.Buffer.from('metadata', 'utf8'), constants_1.METAPLEX_PROGRAM_ID.toBuffer(), mint.toBuffer()], constants_1.METAPLEX_PROGRAM_ID);
    let [editonPda] = web3_js_1.PublicKey.findProgramAddressSync([buffer_1.Buffer.from('metadata', 'utf8'), constants_1.METAPLEX_PROGRAM_ID.toBuffer(), mint.toBuffer(), buffer_1.Buffer.from('edition', 'utf8')], constants_1.METAPLEX_PROGRAM_ID);
    let ata = (0, spl_token_1.getAssociatedTokenAddressSync)(mint, ownerKey);
    let data = new PublishGameData(uri, name, symbol).serialize();
    return new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: ownerKey,
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: mint,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: ata,
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: metadataPda,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: editonPda,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: spl_token_1.TOKEN_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: constants_1.METAPLEX_PROGRAM_ID,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.PublicKey.default,
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: constants_1.PROGRAM_ID,
        data,
    });
}
exports.publishGame = publishGame;
//# sourceMappingURL=instruction.js.map